<?php
namespace wcf\form;
use wcf\system\menu\user\UserMenu;
use wcf\system\WCF;
use wcf\system\user\storage\UserStorageHandler;
use wcf\util\JSON;
use wcf\data\user\UserAction;
use wcf\system\exception\UserInputException;

/**
 * @author	Justin Marsh
 * @license	Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International <https://creativecommons.org/licenses/by-nc-nd/4.0/legalcode>
 */
class TextReplacementModifyForm extends AbstractForm {
    /**
     * @see	\wcf\page\AbstractPage::$loginRequired
     */
    public $loginRequired = true;
    public $neededModules = array('TEXT_REPLACEMENT_ENABLED');
    public $neededPermissions = array('user.board.canDefinePersonalTextReplacements');
    
    private $_textReplacementPairs = array();
    private $_saved = false;

    /**
    * @see	\wcf\page\IPage::readData()
    */
    public function readData() {
        parent::readData();
        $userTextReplacementPairs = array();
        if (!empty(WCF::getUser()->textReplacementPairs)) {
            $userTextReplacementPairs = JSON::decode(WCF::getUser()->textReplacementPairs);
        }
        if (empty($userTextReplacementPairs) || $this->_saved) {
            return;
        }
        $this->_textReplacementPairs = $userTextReplacementPairs;
    }
    
    /**
    * @see	\wcf\form\IForm::readFormParameters()
    */
    public function readFormParameters() {
        parent::readFormParameters();
        if (isset($_POST['textReplaced']) && isset($_POST['textReplacements'])){
            $this->_textReplacementPairs = array_combine($_POST['textReplaced'], $_POST['textReplacements']);
        }
    }

    /**
     * @see	\wcf\form\IForm::validate()
     */
    public function validate() {
        parent::validate();
        $numberOfTextReplacements = count($_POST['textReplacements']);
        $numberOfTextReplaced = count($_POST['textReplaced']);
        // The likelyhood of the arrays not being equal in size in a legitimate situation is minimal
        if ($numberOfTextReplacements !== $numberOfTextReplaced || $numberOfTextReplacements > WCF::getSession()->getPermission('user.board.textReplacementsLimit') || $numberOfTextReplaced > WCF::getSession()->getPermission('user.board.textReplacementsLimit')) {
            throw new UserInputException('', 'userTextReplacementLimitReached');
        }
    }

    /**
     * @see	\wcf\page\IPage::assignVariables()
     */
    public function assignVariables() {
        parent::assignVariables();

        WCF::getTPL()->assign(array(
            'textReplacementPairs' => $this->_textReplacementPairs,
        ));
    }

    /**
     * @see	\wcf\page\IPage::show()
     */
    public function show() {
        UserMenu::getInstance()->setActiveMenuItem('wcf.user.menu.settings.textReplacement');
        parent::show();
    }

    /**
     * @see \wcf\form\IForm::save()
     */
    public function save() {
        parent::save();
        
        $updateInfo = array(
            'textReplacementPairs' => JSON::encode($this->_textReplacementPairs),
        );

        $userAction = new UserAction(array(WCF::getUser()->userID), 'update', array('data' => $updateInfo));
        $userAction->executeAction();

        $this->_saved = true;
        $this->saved();

        // show success message
        WCF::getTPL()->assign('success');
    }
}