<?php
namespace wcf\system\event\listener;
use wcf\system\event\IEventListener;
use wcf\system\WCF;
use wcf\system\user\storage\UserStorageHandler;
use wcf\util\JSON;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;
use wcf\data\user\User;

/**
 * @author	Justin Marsh
 * @license	Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International <https://creativecommons.org/licenses/by-nc-nd/4.0/legalcode>
 */
class TextReplacementListener implements IEventListener{
	public function execute($eventObj, $className, $eventName) {
		if (TEXT_REPLACEMENT_ENABLED) {
			$userTextReplacementPairs = array();
			if (!empty(WCF::getUser()->textReplacementPairs)) {
				$userTextReplacementPairs = JSON::decode(WCF::getUser()->textReplacementPairs, true);
			}

			$textReplaced = array();
			$textReplacements = array();

			if (!WCF::getSession()->getPermission('user.board.canDisableTextReplacement') || WCF::getUser()->getUserOption('enableGeneralTextReplacement')) {
				list($textReplaced, $textReplacements) = $this->_getGlobalTextReplacements();
			}

			if (WCF::getSession()->getPermission('user.board.canDefinePersonalTextReplacements') && !empty($userTextReplacementPairs)) {
				$textReplaced = array_merge($textReplaced, array_keys($userTextReplacementPairs));
				$textReplacements = array_merge($textReplacements, array_values($userTextReplacementPairs));
			}

			$textReplaced = $this->_pregQuoteAndPregFilterArrayValues($textReplaced);
			$newHtml = preg_replace($textReplaced, $textReplacements, $eventObj->getHtml());
			$eventObj->load($eventObj->getHtmlProcessor(), $newHtml);
		}
	}

	private function _pregQuoteAndPregFilterArrayValues($array){
		return array_map(
			function($string) {
				return '/\b' . preg_quote($string, '/') . '\b/i';
			},
			$array
		);
	}

	private function _getGlobalTextReplacements(){
		$textReplaced = array();
		$textReplacements = array();
		$textReplacementPairs = ArrayUtil::trim(explode("\n", StringUtil::unifyNewlines(TEXT_REPLACEMENT_REPLACEMENTS)));
		foreach ($textReplacementPairs as $textReplacementPair){
			$explodedTextReplacementPair = explode(',', $textReplacementPair);
			if (count($explodedTextReplacementPair) === 2) {
				list($textReplaced[], $textReplacements[]) = $explodedTextReplacementPair;
			}
		}
		return array($textReplaced, $textReplacements);
	}
}